;;
;; configuraiton.scm
;;
;; Configuration parameters used by various modules.
;;
;; ISC License
;;
;; Copyright 2023 Brmlab, z.s.
;; Dominik Pantůček <dominik.pantucek@trustica.cz>
;;
;; Permission to use, copy, modify, and/or distribute this software
;; for any purpose with or without fee is hereby granted, provided
;; that the above copyright notice and this permission notice appear
;; in all copies.
;; 
;; THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
;; WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
;; AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
;; CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
;; OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
;; NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
;; CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
;;

(declare (unit configuration))

(module
 configuration
 (
  *current-month*
  
  *member-suspend-max-months*

  *etc-hackerbase*
  
  *members-directory*
  *apikeys-file*
  *jendasap-checked*
  *bank-dir*
  
  load-configuration!
  )

 (import scheme
	 (chicken base)
	 (chicken time)
	 (chicken time posix)
	 (chicken file)
	 (chicken io)
	 month
	 util-parser)

 ;; Current month - if changed, we get the actual state for given month.
 (define *current-month*
   (make-parameter
    (let ((d (seconds->local-time (current-seconds))))
      (list (+ 1900 (vector-ref d 5))
	    (+ (vector-ref d 4) 1)))))

 ;; How long the member can be suspended without any action required?
 (define *member-suspend-max-months* (make-parameter 24))

 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Initial configuration from *etc-hackerbase*

 ;; Where to load initial configuration from
 (define *etc-hackerbase* (make-parameter "/etc/hackerbase"))

 ;; Needed by multiple modules actually
 (define *members-directory* (make-parameter "members"))

 ;; List of bank accounts and Fio API keys (no support for other banks now)
 (define *apikeys-file* (make-parameter "apikey.ntlm"))

 ;; Old last checket transaction id support
 (define *jendasap-checked* (make-parameter "checked.ntlm"))

 ;; Where are the bank CSV files
 (define *bank-dir* (make-parameter #f))

 ;; Loads the configuration file and possibly changes the default
 ;; parameters.
 (define (load-configuration!)
   (when (file-exists? (*etc-hackerbase*))
     (let loop ((lines (read-lines (open-input-file (*etc-hackerbase*)))))
       (when (not (null? lines))
	 (let* ((line (car lines))
		(kv (parser-parse-line
		     (parser-preprocess-line line))))
	   (when (pair? kv)
	     (let ((k (car kv))
		   (v (cdr kv)))
	       (case k
		 ((members-directory) (*members-directory* v))
		 ((apikeys-file) (*apikeys-file* v))
		 ((jendasap-checked) (*jendasap-checked* v))
		 ((bank-dir) (*bank-dir* v)))))
	   (loop (cdr lines)))))))

 )
